@@ -18,7 +18,7 @@ import re
 import netaddr
 from oslo_log import log as logging
 from oslo_utils import uuidutils
-from six import iteritems
+import six
 
 from tacker.common import exceptions as n_exc
 
@@ -93,7 +93,7 @@ def _validate_string_or_none(data, max_len=None):
 
 
 def _validate_string(data, max_len=None):
-    if not isinstance(data, basestring):
+    if not isinstance(data, six.string_types):
         msg = _("'%s' is not a valid string") % data
         LOG.debug(msg)
         return msg
@@ -391,7 +391,7 @@ def _validate_dict_item(key, key_validator, data):
     # TODO(salv-orlando): Structure of dict attributes should be improved
     # to avoid iterating over items
     val_func = val_params = None
-    for (k, v) in iteritems(key_validator):
+    for (k, v) in (key_validator).items():
         if k.startswith('type:'):
             # ask forgiveness, not permission
             try:
@@ -415,7 +415,7 @@ def _validate_dict(data, key_specs=None):
         return
 
     # Check whether all required keys are present
-    required_keys = [key for key, spec in iteritems(key_specs)
+    required_keys = [key for key, spec in (key_specs).items()
                      if spec.get('required')]
 
     if required_keys:
@@ -426,7 +426,7 @@ def _validate_dict(data, key_specs=None):
 
     # Perform validation and conversion of all values
     # according to the specifications.
-    for key, key_validator in [(k, v) for k, v in iteritems(key_specs)
+    for key, key_validator in [(k, v) for k, v in (key_specs).items()
                                if k in data]:
         msg = _validate_dict_item(key, key_validator, data)
         if msg:
@@ -464,7 +464,7 @@ def _validate_non_negative(data, valid_values=None):
 
 
 def convert_to_boolean(data):
-    if isinstance(data, basestring):
+    if isinstance(data, six.string_types):
         val = data.lower()
         if val == "true" or val == "1":
             return True
@@ -485,14 +485,14 @@ def convert_to_int(data):
     try:
         return int(data)
     except (ValueError, TypeError):
-        msg = _("'%s' is not a integer") % data
+        msg = _("'%s' is not an integer") % data
         raise n_exc.InvalidInput(error_message=msg)
 
 
 def convert_kvp_str_to_list(data):
     """Convert a value of the form 'key=value' to ['key', 'value'].
 
-    :raises: n_exc.InvalidInput if any of the strings are malformed
+    :raises n_exc.InvalidInput: if any of the strings are malformed
                                 (e.g. do not contain a key).
     """
     kvp = [x.strip() for x in data.split('=', 1)]
@@ -505,7 +505,7 @@ def convert_kvp_str_to_list(data):
 def convert_kvp_list_to_dict(kvp_list):
     """Convert a list of 'key=value' strings to a dict.
 
-    :raises: n_exc.InvalidInput if any of the strings are malformed
+    :raises n_exc.InvalidInput: if any of the strings are malformed
                                 (e.g. do not contain a key) or if any
                                 of the keys appear more than once.
     """
@@ -517,7 +517,7 @@ def convert_kvp_list_to_dict(kvp_list):
         key, value = convert_kvp_str_to_list(kvp_str)
         kvp_map.setdefault(key, set())
         kvp_map[key].add(value)
-    return dict((x, list(y)) for x, y in iteritems(kvp_map))
+    return dict((x, list(y)) for x, y in (kvp_map).items())
 
 
 def convert_none_to_empty_list(value):
@@ -531,7 +531,7 @@ def convert_none_to_empty_dict(value):
 def convert_to_list(data):
     if data is None:
         return []
-    elif hasattr(data, '__iter__'):
+    elif hasattr(data, '__iter__') and not isinstance(data, six.string_types):
         return list(data)
     else:
         return [data]
