@@ -59,18 +59,27 @@ class VimKeyNotFoundException(exceptions.TackerException):
     message = _("Unable to find key file for VIM %(vim_id)s")
 
 
-class VimDuplicateUrlException(exceptions.TackerException):
-    message = _("VIM with specified auth URL already exists. Cannot register "
-                "duplicate VIM")
+class VimEncryptKeyError(exceptions.TackerException):
+    message = _("Barbican must be enabled for VIM %(vim_id)s")
 
 
 class VimUnsupportedResourceTypeException(exceptions.TackerException):
-    message = _("Resource type %(type) is unsupported by VIM")
+    message = _("Resource type %(type)s is unsupported by VIM")
 
 
 class VimGetResourceException(exceptions.TackerException):
     message = _("Error while trying to issue %(cmd)s to find resource type "
-                "%(type)s")
+                "%(type)s by resource name %(name)s")
+
+
+class VimGetResourceNameNotUnique(exceptions.TackerException):
+    message = _("Getting resource id from VIM with resource name %(name)s "
+                "by %(cmd)s returns more than one")
+
+
+class VimGetResourceNotFoundException(exceptions.TackerException):
+    message = _("Getting resource id from VIM with resource name %(name)s "
+                "by %(cmd)s returns nothing")
 
 
 class VimFromVnfNotFoundException(exceptions.NotFound):
@@ -112,6 +121,11 @@ class VnffgdCpNoForwardingException(exceptions.TackerException):
                 "included in forwarding path")
 
 
+class VnffgdWrongEndpointNumber(exceptions.TackerException):
+    message = _("Specified number_of_endpoints %(number)s is not equal to "
+                "the number of connection_point %(cps)s")
+
+
 class VnffgdInUse(exceptions.InUse):
     message = _('VNFFGD %(vnffgd_id)s is still in use')
 
@@ -130,8 +144,17 @@ class VnffgInvalidMappingException(exceptions.TackerException):
                 "creating/updating VNFFG.")
 
 
-class VnffgVimMappingException(exceptions.TackerException):
-    message = _("VNF Instance VNF %(vnf_id)s does not match VIM ID %(vim_id).")
+class VnffgParamValueFormatError(exceptions.TackerException):
+    message = _("Param values %(param_value)s is not in dict format.")
+
+
+class VnffgTemplateParamParsingException(exceptions.TackerException):
+    message = _("Failed to parse VNFFG Template due to "
+                "missing input param %(get_input)s.")
+
+
+class VnffgParamValueNotUsed(exceptions.TackerException):
+    message = _("Param input %(param_key)s not used.")
 
 
 class VnffgPropertyNotFoundException(exceptions.NotFound):
@@ -176,6 +199,14 @@ class NfpPolicyCriteriaError(exceptions.PolicyCheckError):
     message = _('%(error)s in policy')
 
 
+class NfpPolicyCriteriaIndexError(exceptions.TackerException):
+    message = _('Criteria list can not be empty')
+
+
+class NfpDuplicatePolicyCriteria(exceptions.TackerException):
+    message = _('The %(first_dict)s and %(sec_dict)s are overlapped')
+
+
 class NfpPolicyNotFoundException(exceptions.NotFound):
     message = _('Policy not found in NFP %(nfp)s')
 
@@ -189,7 +220,7 @@ class NfpForwarderNotFoundException(exceptions.NotFound):
 
 
 class NfpRequirementsException(exceptions.TackerException):
-    message = _('VNFD Forwarder %(vnfd) specified more than twice in '
+    message = _('VNFD Forwarder %(vnfd)s specified more than twice in '
                 'requirements path')
 
 
@@ -209,6 +240,14 @@ class ClassifierNotFoundException(exceptions.NotFound):
     message = _('Classifier %(classifier_id)s could not be found')
 
 
+class VnfMappingNotFoundException(exceptions.NotFound):
+    message = _('VNF mapping not found/defined')
+
+
+class VnfMappingNotValidException(exceptions.TackerException):
+    message = _('The %(vnfd)s is not found in constituent VNFDs')
+
+
 class NSDInUse(exceptions.InUse):
     message = _('NSD %(nsd_id)s is still in use')
 
@@ -216,6 +255,25 @@ class NSDInUse(exceptions.InUse):
 class NSInUse(exceptions.InUse):
     message = _('NS %(ns_id)s is still in use')
 
+
+class NoTasksException(exceptions.TackerException):
+    message = _('No tasks to run for %(action)s on %(resource)s')
+
+
+class UpdateChainException(exceptions.TackerException):
+    message = _("%(message)s")
+
+
+class UpdateClassifierException(exceptions.TackerException):
+    message = _("%(message)s")
+
+
+class UpdateVnffgException(exceptions.TackerException):
+    message = _("%(message)s")
+
+
+NAME_MAX_LEN = 255
+
 RESOURCE_ATTRIBUTE_MAP = {
 
     'vims': {
@@ -260,7 +318,7 @@ RESOURCE_ATTRIBUTE_MAP = {
         'name': {
             'allow_post': True,
             'allow_put': True,
-            'validate': {'type:string': None},
+            'validate': {'type:string': NAME_MAX_LEN},
             'is_visible': True,
         },
         'description': {
@@ -325,7 +383,7 @@ RESOURCE_ATTRIBUTE_MAP = {
         'name': {
             'allow_post': True,
             'allow_put': True,
-            'validate': {'type:string': None},
+            'validate': {'type:string': NAME_MAX_LEN},
             'is_visible': True,
         },
         'description': {
@@ -343,6 +401,12 @@ RESOURCE_ATTRIBUTE_MAP = {
             'is_visible': True,
             'default': None,
         },
+        'template_source': {
+            'allow_post': False,
+            'allow_put': False,
+            'is_visible': True,
+            'default': 'onboarded'
+        }
     },
 
     'vnffgs': {
@@ -365,11 +429,12 @@ RESOURCE_ATTRIBUTE_MAP = {
             'allow_put': False,
             'validate': {'type:uuid': None},
             'is_visible': True,
+            'default': None
         },
         'name': {
             'allow_post': True,
             'allow_put': True,
-            'validate': {'type:string': None},
+            'validate': {'type:string': NAME_MAX_LEN},
             'is_visible': True,
         },
         'description': {
@@ -412,6 +477,13 @@ RESOURCE_ATTRIBUTE_MAP = {
             'allow_put': False,
             'is_visible': True,
         },
+        'vnffgd_template': {
+            'allow_post': True,
+            'allow_put': True,
+            'validate': {'type:dict_or_nodata': None},
+            'is_visible': True,
+            'default': None,
+        },
     },
 
     'nfps': {
@@ -564,6 +636,12 @@ RESOURCE_ATTRIBUTE_MAP = {
             'allow_put': False,
             'is_visible': True,
         },
+        'name': {
+            'allow_post': True,
+            'allow_put': True,
+            'validate': {'type:string': NAME_MAX_LEN},
+            'is_visible': True,
+        },
     },
 
     'nsds': {
@@ -584,7 +662,7 @@ RESOURCE_ATTRIBUTE_MAP = {
         'name': {
             'allow_post': True,
             'allow_put': True,
-            'validate': {'type:string': None},
+            'validate': {'type:string': NAME_MAX_LEN},
             'is_visible': True,
         },
         'description': {
@@ -612,6 +690,12 @@ RESOURCE_ATTRIBUTE_MAP = {
             'is_visible': True,
             'default': None,
         },
+        'template_source': {
+            'allow_post': False,
+            'allow_put': False,
+            'is_visible': True,
+            'default': 'onboarded'
+        },
 
     },
 
@@ -633,13 +717,13 @@ RESOURCE_ATTRIBUTE_MAP = {
         'name': {
             'allow_post': True,
             'allow_put': True,
-            'validate': {'type:string': None},
+            'validate': {'type:string': NAME_MAX_LEN},
             'is_visible': True,
         },
         'description': {
             'allow_post': True,
             'allow_put': True,
-            'validate': {'type:string': None},
+            'validate': {'type:string': NAME_MAX_LEN},
             'is_visible': True,
             'default': '',
         },
@@ -665,6 +749,7 @@ RESOURCE_ATTRIBUTE_MAP = {
             'allow_put': False,
             'validate': {'type:uuid': None},
             'is_visible': True,
+            'default': None,
         },
         'vim_id': {
             'allow_post': True,
@@ -683,16 +768,6 @@ RESOURCE_ATTRIBUTE_MAP = {
             'allow_put': False,
             'is_visible': True,
         },
-        'created_at': {
-            'allow_post': False,
-            'allow_put': False,
-            'is_visible': True,
-        },
-        'updated_at': {
-            'allow_post': False,
-            'allow_put': False,
-            'is_visible': True,
-        },
         'attributes': {
             'allow_post': True,
             'allow_put': False,
@@ -708,6 +783,13 @@ RESOURCE_ATTRIBUTE_MAP = {
             'validate': {'type:dict_or_nodata': None},
             'is_visible': True,
         },
+        'nsd_template': {
+            'allow_post': True,
+            'allow_put': False,
+            'validate': {'type:dict_or_nodata': None},
+            'is_visible': True,
+            'default': None,
+        },
     },
 
 }
@@ -772,6 +854,10 @@ class NFVOPluginBase(service_base.NFVPluginBase):
     def create_vim(self, context, vim):
         pass
 
+    @abc.abstractmethod
+    def update_vim(self, context, vim_id, vim):
+        pass
+
     @abc.abstractmethod
     def delete_vim(self, context, vim_id):
         pass
