@@ -36,9 +36,9 @@ from tacker.mistral import mistral_client
 from tacker.nfvo.drivers.vim import abstract_vim_driver
 from tacker.nfvo.drivers.vnffg import abstract_vnffg_driver
 from tacker.nfvo.drivers.workflow import workflow_generator
+from tacker.plugins.common import constants
 from tacker.vnfm import keystone
 
-
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
 
@@ -116,8 +116,11 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
 
         Initialize keystoneclient with provided authentication attributes.
         """
+        verify = 'True' == vim_obj['auth_cred'].get('cert_verify', 'True') \
+                 or False
         auth_url = vim_obj['auth_url']
-        keystone_version = self._validate_auth_url(auth_url)
+        keystone_version = self._validate_auth_url(auth_url=auth_url,
+                                                   verify=verify)
         auth_cred = self._get_auth_creds(keystone_version, vim_obj)
         return self._initialize_keystone(keystone_version, auth_cred)
 
@@ -150,9 +153,9 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
 
         return auth_plugin
 
-    def _validate_auth_url(self, auth_url):
+    def _validate_auth_url(self, auth_url, verify):
         try:
-            keystone_version = self.keystone.get_version(auth_url)
+            keystone_version = self.keystone.get_version(auth_url, verify)
         except Exception as e:
             LOG.error('VIM Auth URL invalid')
             raise nfvo.VimConnectionException(message=str(e))
@@ -331,16 +334,16 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
         :param client_type: openstack client to initialize
         :return: initialized client
         """
+        verify = 'True' == vim_obj.get('cert_verify', 'True') or False
         auth_url = vim_obj['auth_url']
-        keystone_version = self._validate_auth_url(auth_url)
+        keystone_version = self._validate_auth_url(auth_url=auth_url,
+                                                   verify=verify)
         auth_cred = self._get_auth_creds(keystone_version, vim_obj)
         auth_plugin = self._get_auth_plugin(keystone_version, **auth_cred)
         sess = session.Session(auth=auth_plugin)
         return client_type(session=sess)
 
-    def create_flow_classifier(self, name, fc, symmetrical=False,
-                               auth_attr=None):
-        def _translate_ip_protocol(ip_proto):
+    def _translate_ip_protocol(self, ip_proto):
             if ip_proto == '1':
                 return 'icmp'
             elif ip_proto == '6':
@@ -350,28 +353,31 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
             else:
                 return None
 
-        if not auth_attr:
-            LOG.warning("auth information required for n-sfc driver")
-            return None
-
-        if symmetrical:
-            LOG.warning("n-sfc driver does not support symmetrical")
-            raise NotImplementedError('symmetrical chain not supported')
-        LOG.debug('fc passed is %s', fc)
-        sfc_classifier_params = {}
+    def _create_classifier_params(self, fc):
+        classifier_params = {}
         for field in fc:
             if field in FC_MAP:
-                sfc_classifier_params[FC_MAP[field]] = fc[field]
+                classifier_params[FC_MAP[field]] = fc[field]
             elif field == 'ip_proto':
-                protocol = _translate_ip_protocol(str(fc[field]))
+                protocol = self._translate_ip_protocol(str(fc[field]))
                 if not protocol:
                     raise ValueError('protocol %s not supported' % fc[field])
-                sfc_classifier_params['protocol'] = protocol
+                classifier_params['protocol'] = protocol
             else:
                 LOG.warning("flow classifier %s not supported by "
                             "networking-sfc driver", field)
+        return classifier_params
 
+    def create_flow_classifier(self, name, fc, auth_attr=None):
+        if not auth_attr:
+            LOG.warning("auth information required for n-sfc driver")
+            return None
+        fc['name'] = name
+        LOG.debug('fc passed is %s', fc)
+
+        sfc_classifier_params = self._create_classifier_params(fc)
         LOG.debug('sfc_classifier_params is %s', sfc_classifier_params)
+
         if len(sfc_classifier_params) > 0:
             neutronclient_ = NeutronClient(auth_attr)
 
@@ -381,16 +387,12 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
 
         raise ValueError('empty match field for input flow classifier')
 
-    def create_chain(self, name, fc_id, vnfs, symmetrical=False,
+    def create_chain(self, name, fc_ids, vnfs, symmetrical=False,
                      auth_attr=None):
         if not auth_attr:
             LOG.warning("auth information required for n-sfc driver")
             return None
 
-        if symmetrical:
-            LOG.warning("n-sfc driver does not support symmetrical")
-            raise NotImplementedError('symmetrical chain not supported')
-
         neutronclient_ = NeutronClient(auth_attr)
         port_pair_group_list = []
         for vnf in vnfs:
@@ -444,19 +446,119 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
         port_chain['name'] = name + '-port-chain'
         port_chain['description'] = 'port-chain for Tacker VNFFG'
         port_chain['port_pair_groups'] = port_pair_group_list
-        port_chain['flow_classifiers'] = [fc_id]
+        port_chain['flow_classifiers'] = fc_ids
+        if symmetrical:
+            port_chain['chain_parameters'] = {}
+            port_chain['chain_parameters']['symmetric'] = True
         return neutronclient_.port_chain_create(port_chain)
 
     def update_chain(self, chain_id, fc_ids, vnfs,
-                     symmetrical=False, auth_attr=None):
-        # TODO(s3wong): chain can be updated either for
+                     symmetrical=None, auth_attr=None):
+        # (s3wong): chain can be updated either for
         # the list of fc and/or list of port-pair-group
         # since n-sfc driver does NOT track the ppg id
         # it will look it up (or reconstruct) from
         # networking-sfc DB --- but the caveat is that
         # the VNF name MUST be unique
-        LOG.warning("n-sfc driver does not support sf chain update")
-        raise NotImplementedError('sf chain update not supported')
+
+        # TODO(mardim) Currently we figure out which VNF belongs to what
+        # port-pair-group or port-pair through the name of VNF.
+        # This is not the best approach. The best approach for the future
+        # propably is to maintain in the database the ID of the
+        # port-pair-group and port-pair that VNF belongs to so we can
+        # implemement the update in a more robust way.
+
+        if not auth_attr:
+            LOG.warning("auth information required for n-sfc driver")
+            return None
+
+        neutronclient_ = NeutronClient(auth_attr)
+        new_ppgs = []
+        updated_port_chain = dict()
+        pc_info = neutronclient_.port_chain_show(chain_id)
+        if set(fc_ids) != set(pc_info['port_chain']['flow_classifiers']):
+            updated_port_chain['flow_classifiers'] = fc_ids
+        old_ppgs = pc_info['port_chain']['port_pair_groups']
+        old_ppgs_dict = {neutronclient_.
+                    port_pair_group_show(ppg_id)['port_pair_group']['name'].
+                    split('-')[0]: ppg_id for ppg_id in old_ppgs}
+        past_ppgs_dict = old_ppgs_dict.copy()
+        try:
+            for vnf in vnfs:
+                port_pair_group = {}
+                port_pair = {}
+                if vnf['name'] in old_ppgs_dict:
+                    old_ppg_id = old_ppgs_dict.pop(vnf['name'])
+                    new_ppgs.append(old_ppg_id)
+                else:
+                    port_pair_group['name'] = vnf['name'] + '-port-pair-group'
+                    port_pair_group['description'] = \
+                        'port pair group for %s' % vnf['name']
+                    port_pair_group['port_pairs'] = []
+
+                    if CONNECTION_POINT not in vnf:
+                        LOG.warning("Chain update failed due to missing "
+                                    "connection point info in VNF "
+                                    "%(vnfname)s", {'vnfname': vnf['name']})
+                        raise nfvo.UpdateChainException(
+                            message="Connection point not found")
+                    cp_list = vnf[CONNECTION_POINT]
+                    num_cps = len(cp_list)
+                    if num_cps != 1 and num_cps != 2:
+                        LOG.warning("Chain update failed due to wrong number "
+                                    "of connection points: expected [1 | 2],"
+                                    "got %(cps)d", {'cps': num_cps})
+                        raise nfvo.UpdateChainException(
+                            message="Invalid number of connection points")
+                    port_pair['name'] = vnf['name'] + '-connection-points'
+                    port_pair['description'] = 'port pair for %s' % vnf['name']
+                    if num_cps == 1:
+                        port_pair['ingress'] = cp_list[0]
+                        port_pair['egress'] = cp_list[0]
+                    else:
+                        port_pair['ingress'] = cp_list[0]
+                        port_pair['egress'] = cp_list[1]
+                    port_pair_id = neutronclient_.port_pair_create(port_pair)
+                    if not port_pair_id:
+                        LOG.warning("Chain update failed due to port pair "
+                                    "creation failed for "
+                                    "vnf %(vnf)s", {'vnf': vnf['name']})
+                        raise nfvo.UpdateChainException(
+                            message="Failed to create port-pair")
+                    port_pair_group['port_pairs'].append(port_pair_id)
+                    port_pair_group_id = \
+                        neutronclient_.port_pair_group_create(port_pair_group)
+                    if not port_pair_group_id:
+                        LOG.warning("Chain update failed due to port pair "
+                                    "group creation failed for vnf "
+                                    "%(vnf)s", {'vnf': vnf['name']})
+                        for pp_id in port_pair_group['port_pairs']:
+                            neutronclient_.port_pair_delete(pp_id)
+                        raise nfvo.UpdateChainException(
+                            message="Failed to create port-pair-group")
+                    new_ppgs.append(port_pair_group_id)
+        except nfvo.UpdateChainException as e:
+            self._delete_ppgs_and_pps(neutronclient_, new_ppgs, past_ppgs_dict)
+            raise e
+
+        updated_port_chain['port_pair_groups'] = new_ppgs
+        updated_port_chain['flow_classifiers'] = fc_ids
+        try:
+            pc_id = neutronclient_.port_chain_update(chain_id,
+                                                     updated_port_chain)
+        except (nc_exceptions.BadRequest, nfvo.UpdateChainException) as e:
+            self._delete_ppgs_and_pps(neutronclient_, new_ppgs, past_ppgs_dict)
+            raise e
+        for ppg_name in old_ppgs_dict:
+            ppg_info = neutronclient_. \
+                port_pair_group_show(old_ppgs_dict[ppg_name])
+            neutronclient_.port_pair_group_delete(old_ppgs_dict[ppg_name])
+            port_pairs = ppg_info['port_pair_group']['port_pairs']
+            if port_pairs and len(port_pairs):
+                for j in xrange(0, len(port_pairs)):
+                    pp_id = port_pairs[j]
+                    neutronclient_.port_pair_delete(pp_id)
+        return pc_id
 
     def delete_chain(self, chain_id, auth_attr=None):
         if not auth_attr:
@@ -466,28 +568,89 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
         neutronclient_ = NeutronClient(auth_attr)
         neutronclient_.port_chain_delete(chain_id)
 
-    def update_flow_classifier(self, fc_id, fc,
-                               symmetrical=False, auth_attr=None):
+    def update_flow_classifier(self, chain_id, fc, auth_attr=None):
         if not auth_attr:
             LOG.warning("auth information required for n-sfc driver")
             return None
 
-        if symmetrical:
-            LOG.warning("n-sfc driver does not support symmetrical")
-            raise NotImplementedError('symmetrical chain not supported')
-
-        # for now, the only parameters allowed for flow-classifier-update
-        # is 'name' and/or 'description'
-
-        sfc_classifier_params = {}
-        sfc_classifier_params['name'] = fc['name']
-        sfc_classifier_params['description'] = fc['description']
-
-        LOG.debug('sfc_classifier_params is %s', sfc_classifier_params)
+        fc_id = fc.pop('instance_id')
+        fc_status = fc.pop('status')
+        match_dict = fc.pop('match')
+        fc.update(match_dict)
 
+        sfc_classifier_params = self._create_classifier_params(fc)
         neutronclient_ = NeutronClient(auth_attr)
-        return neutronclient_.flow_classifier_update(fc_id,
-                                                     sfc_classifier_params)
+        if fc_status == constants.PENDING_UPDATE:
+            fc_info = neutronclient_.flow_classifier_show(fc_id)
+            for field in sfc_classifier_params:
+                # If the new classifier is the same with the old one then
+                # no change needed.
+                if (fc_info['flow_classifier'].get(field) is not None) and \
+                        (sfc_classifier_params[field] == fc_info[
+                            'flow_classifier'][field]):
+                    continue
+
+                # If the new classifier has different match criteria
+                # with the old one then we strip the classifier from
+                # the chain we delete the old classifier and we create
+                # a new one with the same name as before but with different
+                # match criteria. We are not using the flow_classifier_update
+                # from the n-sfc because it does not support match criteria
+                # update for an existing classifier yet.
+                else:
+                    try:
+                        self._dissociate_classifier_from_chain(chain_id,
+                                                               [fc_id],
+                                                               neutronclient_)
+                    except Exception as e:
+                        raise e
+                    fc_id = neutronclient_.flow_classifier_create(
+                        sfc_classifier_params)
+                    if fc_id is None:
+                        raise nfvo.UpdateClassifierException(
+                            message="Failed to update classifiers")
+                    break
+
+        # If the new classifier is completely different from the existing
+        # ones (name and match criteria) then we just create it.
+        else:
+            fc_id = neutronclient_.flow_classifier_create(
+                sfc_classifier_params)
+            if fc_id is None:
+                raise nfvo.UpdateClassifierException(
+                    message="Failed to update classifiers")
+
+        return fc_id
+
+    def _dissociate_classifier_from_chain(self, chain_id, fc_ids,
+                                          neutronclient):
+            pc_info = neutronclient.port_chain_show(chain_id)
+            current_fc_list = pc_info['port_chain']['flow_classifiers']
+            for fc_id in fc_ids:
+                current_fc_list.remove(fc_id)
+            pc_id = neutronclient.port_chain_update(chain_id,
+                    {'flow_classifiers': current_fc_list})
+            if pc_id is None:
+                raise nfvo.UpdateClassifierException(
+                    message="Failed to update classifiers")
+            for fc_id in fc_ids:
+                try:
+                    neutronclient.flow_classifier_delete(fc_id)
+                except ValueError as e:
+                    raise e
+
+    def remove_and_delete_flow_classifiers(self, chain_id, fc_ids,
+                                           auth_attr=None):
+        if not auth_attr:
+            LOG.warning("auth information required for n-sfc driver")
+            raise EnvironmentError('auth attribute required for'
+                                   ' networking-sfc driver')
+        neutronclient_ = NeutronClient(auth_attr)
+        try:
+            self._dissociate_classifier_from_chain(chain_id, fc_ids,
+                                                   neutronclient_)
+        except Exception as e:
+            raise e
 
     def delete_flow_classifier(self, fc_id, auth_attr=None):
         if not auth_attr:
@@ -519,33 +682,59 @@ class OpenStack_Driver(abstract_vim_driver.VimAbstractDriver,
         return {'id': workflow[0].id, 'input': wg.get_input_dict()}
 
     def execute_workflow(self, workflow, auth_dict=None):
-        return self.get_mistral_client(auth_dict)\
+        return self.get_mistral_client(auth_dict) \
             .executions.create(
-                workflow_identifier=workflow['id'],
-                workflow_input=workflow['input'],
-                wf_params={})
+            workflow_identifier=workflow['id'],
+            workflow_input=workflow['input'],
+            wf_params={})
 
     def get_execution(self, execution_id, auth_dict=None):
-        return self.get_mistral_client(auth_dict)\
+        return self.get_mistral_client(auth_dict) \
             .executions.get(execution_id)
 
     def delete_execution(self, execution_id, auth_dict=None):
-        return self.get_mistral_client(auth_dict).executions\
+        return self.get_mistral_client(auth_dict).executions \
             .delete(execution_id)
 
     def delete_workflow(self, workflow_id, auth_dict=None):
-        return self.get_mistral_client(auth_dict)\
+        return self.get_mistral_client(auth_dict) \
             .workflows.delete(workflow_id)
 
+    def _delete_ppgs_and_pps(self, neutronclient, new_ppgs, past_ppgs_dict):
+        if new_ppgs:
+            for item in new_ppgs:
+                if item not in past_ppgs_dict.values():
+                    new_ppg_info = neutronclient.port_pair_group_show(
+                        item)
+                    neutronclient.port_pair_group_delete(item)
+                    new_port_pairs = new_ppg_info['port_pair_group'][
+                        'port_pairs']
+                    if new_port_pairs and len(new_port_pairs):
+                        for j in xrange(0, len(new_port_pairs)):
+                            new_pp_id = new_port_pairs[j]
+                            neutronclient.port_pair_delete(new_pp_id)
+
 
 class NeutronClient(object):
     """Neutron Client class for networking-sfc driver"""
 
     def __init__(self, auth_attr):
-        auth = identity.Password(**auth_attr)
-        sess = session.Session(auth=auth)
+        auth_cred = auth_attr.copy()
+        verify = 'True' == auth_cred.pop('cert_verify', 'True') or False
+        auth = identity.Password(**auth_cred)
+        sess = session.Session(auth=auth, verify=verify)
         self.client = neutron_client.Client(session=sess)
 
+    def flow_classifier_show(self, fc_id):
+        try:
+            fc = self.client.show_flow_classifier(fc_id)
+            if fc is None:
+                raise ValueError('classifier %s not found' % fc_id)
+            return fc
+        except nc_exceptions.NotFound:
+            LOG.error('classifier %s not found', fc_id)
+            raise ValueError('classifier %s not found' % fc_id)
+
     def flow_classifier_create(self, fc_dict):
         LOG.debug("fc_dict passed is {fc_dict}".format(fc_dict=fc_dict))
         fc = self.client.create_flow_classifier({'flow_classifier': fc_dict})
@@ -636,3 +825,38 @@ class NeutronClient(object):
         except nc_exceptions.NotFound:
             LOG.warning('port chain %s not found', port_chain_id)
             raise ValueError('port chain %s not found' % port_chain_id)
+
+    def port_chain_update(self, port_chain_id, port_chain):
+        try:
+            pc = self.client.update_port_chain(port_chain_id,
+                                    {'port_chain': port_chain})
+        except nc_exceptions.BadRequest as e:
+            LOG.warning('update port chain returns %s', e)
+            raise ValueError(str(e))
+        if pc and len(pc):
+            return pc['port_chain']['id']
+        else:
+            raise nfvo.UpdateChainException(message="Failed to update "
+                                                    "port-chain")
+
+    def port_chain_show(self, port_chain_id):
+        try:
+            port_chain = self.client.show_port_chain(port_chain_id)
+            if port_chain is None:
+                raise ValueError('port chain %s not found' % port_chain_id)
+
+            return port_chain
+        except nc_exceptions.NotFound:
+            LOG.error('port chain %s not found', port_chain_id)
+            raise ValueError('port chain %s not found' % port_chain_id)
+
+    def port_pair_group_show(self, ppg_id):
+        try:
+            port_pair_group = self.client.show_port_pair_group(ppg_id)
+            if port_pair_group is None:
+                raise ValueError('port pair group %s not found' % ppg_id)
+
+            return port_pair_group
+        except nc_exceptions.NotFound:
+            LOG.warning('port pair group %s not found', ppg_id)
+            raise ValueError('port pair group %s not found' % ppg_id)
