@@ -87,20 +87,16 @@ class TestOpenStack(base.TestCase):
 
     def _get_expected_fields(self):
         return {'stack_name':
-                'tacker.vnfm.infra_drivers.openstack.openstack_OpenStack'
-                '-eb84260e-5ff7-4332-b032-50a14d6c1123', 'template':
-                self.hot_template}
+                'test_openwrt_eb84260e-5ff7-4332-b032-50a14d6c1123',
+                'template': self.hot_template}
 
     def _get_expected_fields_user_data(self):
         return {'stack_name':
-                'tacker.vnfm.infra_drivers.openstack.openstack_OpenStack'
-                '-18685f68-2b2a-4185-8566-74f54e548811',
+                'test_userdata_18685f68-2b2a-4185-8566-74f54e548811',
                 'template': self.hot_param_template}
 
     def _get_expected_fields_ipaddr_data(self):
-        return {'stack_name':
-                'tacker.vnfm.infra_drivers.openstack.openstack_OpenStack'
-                '-d1337add-d5a1-4fd4-9447-bb9243c8460b',
+        return {'stack_name': 'test_ip_d1337add-d5a1-4fd4-9447-bb9243c8460b',
                 'template': self.hot_ipparam_template}
 
     def _get_expected_vnf_wait_obj(self, param_values=''):
@@ -187,9 +183,7 @@ class TestOpenStack(base.TestCase):
 
     def _get_expected_fields_tosca(self, template):
         return {'stack_name':
-                'tacker.vnfm.infra_drivers.openstack.openstack_OpenStack'
-                '-eb84260e'
-                '-5ff7-4332-b032-50a14d6c1123',
+                'test_openwrt_eb84260e-5ff7-4332-b032-50a14d6c1123',
                 'template': _get_template(template)}
 
     def _get_expected_tosca_vnf(self,
