@@ -38,7 +38,8 @@ class Keystone(object):
 
     def get_version(self, base_url=None, verify=True):
         try:
-            keystone_client = client.Client(auth_url=base_url, verify=verify)
+            keystone_client = client.Client(auth_url=base_url,
+                                            verify=verify)
         except exceptions.ConnectionError:
             raise
         return keystone_client.version
